import chimera, numpy
from chimera import runCommand
from numpy import array, arccos, linalg, zeros

from StructBio.StructureOverlap.overlapper import Overlapper
from StructBio.Scenographics.solids import Solids
from StructBio.LeastSquares.inertialAxes import InertialAxes
from StructBio.SSEs.tubesForSSEs import TubesForSSEs

def getAtom(prot, resPosition, atomName):
    for r in prot.residues:
        if r.id.position == resPosition:
            return r.findAtom(atomName)
        
#==============================================================================
# NOTE: This code is only suitable for mechanosensitive channels 2OAU and 2VV5.
# Note also that the primary axis derived from the computation of the inertial axes
# will be the axis of symmetry because of the overall shape of the protein.
# This is because the longest dimension of the molecule (height of the channel)
# is along this axis of symmetry.
#==============================================================================

def mainAxisAndDisk(prot):
    # Put in three extra visual components 
    # The next two visual components will need some calculations that define three points
    # along the main axis of the protein. We start by generating the inertial axis for the protein.
    caAtoms_L = []
    for r in prot.residues:
        if r.findAtom("CA"):
            caAtoms_L.append(r.findAtom("CA"))
    mainAxis = InertialAxes(caAtoms_L) # NB: This, by itself, does not produce any visual component.
    # We project three atom coordinates onto this inertial axes to get points needed to specify
    # the remaining visual components:
    # (Residue positions 0, 35, and 250 were determined by visual inspection.)
    tubeTopPt = mainAxis.projectionOnAxis(prot.findResidue(0).findAtom("CA"))   
    diskCenterPt = mainAxis.projectionOnAxis(prot.findResidue(35).findAtom("CA"))  
    tubeBotPt = mainAxis.projectionOnAxis(prot.findResidue(250).findAtom("CA"))     

    # 1) Put in the tube for the main axis:
    axisTubeDisp = Solids("Tube Display")
    axisTubeDisp.addTube(tubeBotPt, tubeTopPt, 1.0, (.2, .3, .7))
    axisTubeDisp.display()

    # 2) Put in the disk perpendicular to the main axis:
    diskDisp = Solids("Disk Display")
    diskDisp.addDisk(diskCenterPt, mainAxis.primaryAxis, 0.2, 40., (0.2, 0.6, .4))
    diskDisp.display()
    return
    

#==============================================================================
# Mainline:
#==============================================================================
protP = chimera.openModels.open("2OAU", type = "PDB")[0]
protQ = chimera.openModels.open("2VV5", type = "PDB")[0]

mainAxisAndDisk(protQ)

runCommand("set bg_color white; sel #0-1:.A; sel invert; delete sel; ~sel")
runCommand("modelcolour pink #0; modelcolour turquoise #1")

overlap_L = []

for pos in range(183, 278):   
    overlap_L.append([getAtom(protP, pos, "CA"), getAtom(protQ, pos, "CA")])
                                  
overlapper = Overlapper(protP)
overlapper.moveToOverlap(overlap_L)


# Put tubes into all the helices:
#       Note high tolerance value to avoid recursive tubing.
helixTubesP = TubesForSSEs(protP, tolerance = 4.0, minLength = 6, chainId = "A", strands = False,
                           helixTubeRadius = 0.5, helixTubeColor = (1.000, 0.44, 0.44))
helixTubesQ = TubesForSSEs(protQ, tolerance = 4.0, minLength = 6, chainId = "A", strands = False,
                           helixTubeRadius = 0.5, helixTubeColor = (0.44, 1.00, 0.44))

runCommand("window")
 
print "RMSD for overlap: %6.4f" % overlapper.rmsdForOverlap()


