import chimera, numpy
from chimera import runCommand, Point
from numpy import array, zeros, dot, linalg
from StructBio.StructureOverlap.overlapper import Overlapper

# Global variable:
dataFileDirectoryPath = "Ex4datafiles/"

#-------------------------------------------------------------------------
def getAlphaCarbs(prot, chainID, residuesInAlignment_L):
    # This routine works on the protein object designated by prot.
    # It inspects the chain designated by the character chainID and it
    # looks for residues that have the residue identifiers stored in the
    # residuesInAlignment_L list which contains character strings such as "CYS 191".
    # When a match is detected the alpha carbon atom at that residue 
    # is appended to the return_L list to be returned.

    return_L = []
    for rA in residuesInAlignment_L:
        fullResIdentifier = rA + chainID
        for rProt in prot.residues:
            if fullResIdentifier == rProt.type + "%4d" % (rProt.id.position) + rProt.id.chainId:
                return_L.append(rProt.findAtom("CA"))
    return return_L

#==============================================================================
# ApoProt CLASS
#==============================================================================
class ApoProt(object):
    def __init__(self, apo_FileName):        
        apo_PrincipalResidues_L = ["THR  89", "TYR  91", "SER  92", "VAL 106", "LYS 141", "PHE 142",
                                   "GLY 143", "SER 144", "TRP 145", "VAL 146", "TYR 147", "GLN 184",
                                   "TYR 186", "SER 187", "CYS 188", "CYS 189", "TYR 193", "ASP 195"]
        apo_ComplementResidues_L = ["THR  34", "GLN  36", "TYR  53", "GLN  55", "ARG  77", "VAL 106",
                                    "MET 114", "ILE 116", "ASP 162", "SER 164", "SER 165"]
        apo_C_loopResidues_L = ["TYR 186", "SER 187", "CYS 188", "CYS 189", "PRO 190",
                                "GLU 191", "PRO 192", "TYR 193", "ILE 194"]

        apo_PathName = dataFileDirectoryPath + apo_FileName + ".pdb"
        self.prot = chimera.openModels.open(apo_PathName, type="PDB")[0]
        chainApo_Principals = apo_PathName[-6]
        chainApo_Cmplements = apo_PathName[-5]        
        
        # Build list of alpha carbon atoms to be used in structural overlap:
        self.alphaCarbs_Apo_L = getAlphaCarbs(self.prot, chainApo_Principals, apo_PrincipalResidues_L)
        self.alphaCarbs_Apo_L.extend(getAlphaCarbs(self.prot, chainApo_Cmplements, apo_ComplementResidues_L))

        # Build list of alpha carbon atoms in C loop:
        self.alphaCarbs_ApoC_loop_L = getAlphaCarbs(self.prot, chainApo_Principals, apo_C_loopResidues_L)

#==============================================================================
# HoloProt CLASS
#==============================================================================
class HoloProt(object):
    def __init__(self, holoFileName, ligandName, modelNumber):
     
        holoPrincipalResidues_L = ["THR  91", "TYR  93", "SER  94", "VAL 108", "LYS 143", "PHE 144",
                                   "GLY 145", "SER 146", "TRP 147", "VAL 148", "TYR 149", "GLN 186",
                                   "TYR 188", "SER 189", "CYS 190", "CYS 191", "TYR 195", "ASP 197"]
        holoComplementResidues_L = ["THR  36", "GLN  38", "TYR  55", "GLN  57", "ARG  79", "VAL 108",
                                    "MET 116", "ILE 118", "ASP 164", "SER 166", "SER 167"]
        holoC_loopResidues_L = ["TYR 188", "SER 189", "CYS 190", "CYS 191", "PRO 192",
                                "GLU 193", "PRO 194", "TYR 195", "ILE 196"]

        self.ligandName = ligandName
        self.modelNumber = modelNumber

        holoPathName = dataFileDirectoryPath + holoFileName + ".pdb"
        self.prot = chimera.openModels.open(holoPathName, type="PDB")[0]
        runCommand("sel ligand; represent sphere sel; ~sel")
        chainHoloPrincipals = holoFileName[5]
        chainHoloCmplements = holoFileName[6]

        # Build list of alpha carbon atoms to be used in structural overlap:
        self.alphaCarbsHolo_L = getAlphaCarbs(self.prot, chainHoloPrincipals, holoPrincipalResidues_L)
        self.alphaCarbsHolo_L.extend(getAlphaCarbs(self.prot, chainHoloCmplements, holoComplementResidues_L))

        # Build list of alpha carbon atoms in C loop:
        self.alphaCarbsHoloC_loop_L = getAlphaCarbs(self.prot, chainHoloPrincipals, holoC_loopResidues_L)

        self.overLapper = Overlapper(self.prot)

    def overlapWithApo(self, apoObj):
        self.overLapper.moveToOverlap(zip(self.alphaCarbsHolo_L, apoObj.alphaCarbs_Apo_L))

    def averageDeviaionCloop(self, apoObj):
        # Calculate average deviation in C loop:
        sumDeviations = 0.0
        numResInC_loop = len(self.alphaCarbsHoloC_loop_L)
        for ix in range(numResInC_loop):
            sumDeviations += numpy.linalg.norm(apoObj.alphaCarbs_ApoC_loop_L[ix].coord() - self.alphaCarbsHoloC_loop_L[ix].coord())
        return sumDeviations/numResInC_loop

    def hideMe(self):
        runCommand("sel #" + str(self.modelNumber) + "; ~display sel; ~ribbon sel; ~sel")

    def seeMe(self):
        runCommand("sel #" + str(self.modelNumber) + "; ~display sel; ribbon sel; ~sel")
#==============================================================================
# Mainline:
#==============================================================================
# Initialize various lists:
pdbFiles_L = [("2BYR_AE", "methyllycaconitine"),
              ("3PMZ_ED", "d-tubocurarine"),
              ("2WNC_AE", "tropisetron"),
              ("3C79_AE", "imidacloprid"),
              ("2PGZ_AB", "cocaine"),
              ("2BYQ_AE", "epibatidine"),
              ("2BYS_AE", "lobeline")]

runCommand("set bg_color white")

apoProtObj = ApoProt("2Y7Y_EA")

raw_input("You may put the apo protein in its best position.  Then hit Enter to continue.")
holoProtObj_D = {}
tableEntries_L = []
modNumber = 0
for tup in pdbFiles_L:
    holoFileName = tup[0]
    ligandName = tup[1]
    modNumber += 1    
    holoProtObj_D[ligandName] = HoloProt(holoFileName, ligandName, modNumber)
    holoProtObj_D[ligandName].overlapWithApo(apoProtObj)
    raw_input("Overlap with " + ligandName + ". Hit Enter to continue.")
    holoProtObj_D[ligandName].hideMe()
    tableEntries_L.append((ligandName, holoProtObj_D[ligandName].averageDeviaionCloop(apoProtObj)))

runCommand("focus")

print "\n"
print "Ligand".ljust(20) + "Average Deviation".ljust(20)
print " ".ljust(20) + "of C loop atoms:".ljust(20)
for tup in tableEntries_L:
    print tup[0].ljust(20) + '%7.4f' % tup[1]
print "\n"

raw_input("Hit Enter to see overlap with all holo proteins.")
for holoP in holoProtObj_D.keys():
    holoProtObj_D[holoP].seeMe()
