import chimera, numpy as np
from chimera import runCommand
from StructBio.LeastSquares.inertialAxes import InertialAxes
from StructBio.Scenographics.ellipsoid import Ellipsoid
from StructBio.Scenographics.solids import Solids
from StructBio.StructureOverlap.overlapper import Overlapper

#-------------------------------------------------------------------------
def setUpAxes(inertialAxesObj):
    colors_L = [(1.,0.,0.), (0.,1.,0.), (0.,0.,1.)]
    axes = Solids()
    for i in range(3):
        ends = inertialAxesObj.min_maxProjection(i)
        axes.addSpindle(inertialAxesObj.inertialAxes_L[i] * ends[0] + inertialAxesObj.centroid,
                      inertialAxesObj.inertialAxes_L[i] * ends[1] + inertialAxesObj.centroid,
                      .3, colors_L[i])
    axes.display()


#-------------------------------------------------------------------------
def getResiduesInDomain(prot, domainDescriptor):
    # The domainDescriptor is a list of tuples.  Each tuple has the start and end
    # positions of consecutive residues contributing to the domain.
    residues_L = []
    resPositions_L = []
    for pair in domainDescriptor: resPositions_L.extend(range(pair[0], pair[1] + 1))
    for r in prot.residues:
        if r.id.position in resPositions_L: residues_L.append(r)
    return residues_L

#-------------------------------------------------------------------------
def getEllipsoidDimensions(axesObj):
    ends = axesObj.min_maxProjection(0)
    length = 0.5*(abs(ends[0]) + abs(ends[1]))
    ends = axesObj.min_maxProjection(1)
    width = 0.5*(abs(ends[0]) + abs(ends[1]))
    ends = axesObj.min_maxProjection(2)
    height = 0.5*(abs(ends[0]) + abs(ends[1]))
    return (length, width, height)


#==============================================================================
# Mainline
#==============================================================================

runCommand("set bg_color white")

apo_ProtID = "1EX6"
apo_Chain = "B"
holoProtID = "1EX7"
holoChain = "A"

holoDomainD1 = [(1, 32), (84, 186)]
holoDomainD2 = [(33, 83)]

apo_DomainD1 = [(201, 232), (284, 386)]
apo_DomainD2 = [(233, 283)]

commandLine = "sel #;"

apo_Prot = chimera.openModels.open(apo_ProtID, type="PDB")[0]
commandLine += " ~sel #" + str(apo_Prot.id) + ":." + apo_Chain + ";"

holoProt = chimera.openModels.open(holoProtID, type="PDB")[0]
commandLine += " ~sel #" + str(holoProt.id) + ":." + holoChain + ";"

commandLine += " delete sel; ~sel"
runCommand(commandLine)

# Find residues to be overlapped (in domainD1 for each protein).
apo_ResiduesForOverlap_L = getResiduesInDomain(apo_Prot, apo_DomainD1)
holoResiduesForOverlap_L = getResiduesInDomain(holoProt, holoDomainD1)

# Do the overlap:
overlap_L = [(ra.findAtom("CA"), rh.findAtom("CA"))
             for (ra, rh) in zip(apo_ResiduesForOverlap_L, holoResiduesForOverlap_L)]
overlapper = Overlapper(apo_Prot)
overlapper.moveToOverlap(overlap_L)

runCommand("focus")


apo_atoms_L = []
for r in apo_ResiduesForOverlap_L: apo_atoms_L.extend(r.atoms)
apo_AxesObj = InertialAxes(apo_atoms_L, massWeighted = True)

setUpAxes(apo_AxesObj)
(length, width, height) = getEllipsoidDimensions(apo_AxesObj)
d1_ellipsoid = Ellipsoid(length, width, height, apo_AxesObj.centroid,
                         apo_AxesObj.inertialAxes_L, 32, 16)
d1_ellipsoid.displayMesh((0.5, 0.5, 0.5))


apo_D2Residues_L = getResiduesInDomain(apo_Prot, apo_DomainD2)
apo_D2atoms_L = []
for r in apo_D2Residues_L: apo_D2atoms_L.extend(r.atoms)
apo_D2AxesObj = InertialAxes(apo_D2atoms_L, massWeighted = True)

setUpAxes(apo_D2AxesObj)
(length, width, height) = getEllipsoidDimensions(apo_D2AxesObj)
apo_d2_ellipsoid = Ellipsoid(length, width, height, apo_D2AxesObj.centroid,
                         apo_D2AxesObj.inertialAxes_L, 32, 16)
apo_d2_ellipsoid.displayMesh((1.0, 0.0, 0.0))


holoD2Residues_L = getResiduesInDomain(holoProt, holoDomainD2)
holoD2atoms_L = []
for r in holoD2Residues_L: holoD2atoms_L.extend(r.atoms)
holoD2AxesObj = InertialAxes(holoD2atoms_L, massWeighted = True)

setUpAxes(holoD2AxesObj)
(length, width, height) = getEllipsoidDimensions(holoD2AxesObj)
holod2_ellipsoid = Ellipsoid(length, width, height, holoD2AxesObj.centroid,
                         holoD2AxesObj.inertialAxes_L, 32, 16)
holod2_ellipsoid.displayMesh((0.0, 1.0, 0.0))









